@@ -15,6 +15,7 @@ from djangoldp_component.models import Component, Package
 class ApplicationTemplate(Model):
     friendly_name = models.CharField(max_length=255, blank=True, null=True)
     short_description = models.CharField(max_length=255, blank=True, null=True)
+    repository = models.CharField(max_length=255, blank=True, null=True)
     creator = models.ForeignKey(
         get_user_model(),
         related_name="templates",
@@ -108,7 +109,7 @@ class TemplateRequiredComponent(Model):
         if (
             not self.pk
             and TemplateRequiredComponent.objects.filter(
-                component=self.component, package=self.package
+                component=self.component, template=self.template
             ).exists()
         ):
             return
@@ -303,7 +304,7 @@ class ApplicationComponent(Model):
         anonymous_perms = ["view"]
         authenticated_perms = ["inherit"]
         container_path = "application-components/"
-        nested_fields = ["component"]
+        nested_fields = ["component", "parameters"]
         owner_field = "application__creator"
         owner_perms = ["inherit", "change", "delete"]
         rdf_context = {
@@ -311,7 +312,7 @@ class ApplicationComponent(Model):
             "component": "sib:component",
         }
         rdf_type = "sib:dependency"
-        serializer_fields = ["@id", "component"]
+        serializer_fields = ["@id", "component", "parameters"]
         superuser_perms = ["inherit"]
         verbose_name = _("application component")
         verbose_name_plural = _("application components")
@@ -320,7 +321,7 @@ class ApplicationComponent(Model):
         if (
             not self.pk
             and ApplicationComponent.objects.filter(
-                component=self.component, package=self.package
+                component=self.component, application=self.application
             ).exists()
         ):
             return
@@ -328,6 +329,133 @@ class ApplicationComponent(Model):
         super(ApplicationComponent, self).save(*args, **kwargs)
 
 
+class ApplicationComponentParameter(Model):
+    component = models.ForeignKey(
+        ApplicationComponent,
+        on_delete=models.CASCADE,
+        related_name="parameters",
+        null=True,
+        blank=True,
+    )
+    key = models.CharField(max_length=255, blank=True, null=True)
+    value = models.CharField(max_length=255, blank=True, null=True)
+
+    def __str__(self):
+        try:
+            return "{} -> {} ({})".format(
+                self.component.application.friendly_name, self.component.component.friendly_name, self.urlid
+            )
+        except:
+            return self.urlid
+
+    class Meta(Model.Meta):
+        anonymous_perms = ["view"]
+        authenticated_perms = ["inherit"]
+        container_path = "application-component-parameters/"
+        owner_field = "component__application__creator"
+        owner_perms = ["inherit", "change", "delete"]
+        rdf_context = {
+            "component": "sib:dependency",
+            "key": "sib:key",
+            "value": "sib:value",
+        }
+        rdf_type = "sib:parameter"
+        serializer_fields = ["@id", "key", "value"]
+        superuser_perms = ["inherit"]
+        verbose_name = _("component parameter")
+        verbose_name_plural = _("component parameters")
+
+
+class ApplicationPackage(Model):
+    application = models.ForeignKey(
+        Application,
+        on_delete=models.CASCADE,
+        related_name="packages",
+        null=True,
+        blank=True,
+    )
+    package = models.ForeignKey(
+        Package,
+        on_delete=models.CASCADE,
+        related_name="applications",
+        null=True,
+        blank=True,
+    )
+
+    def __str__(self):
+        try:
+            return "{} + {} ({})".format(
+                self.application.friendly_name, self.package.friendly_name, self.urlid
+            )
+        except:
+            return self.urlid
+
+    class Meta(Model.Meta):
+        anonymous_perms = ["view"]
+        authenticated_perms = ["inherit"]
+        container_path = "application-packages/"
+        nested_fields = ["package", "parameters"]
+        owner_field = "application__creator"
+        owner_perms = ["inherit", "change", "delete"]
+        rdf_context = {
+            "application": "sib:application",
+            "package": "sib:package",
+        }
+        rdf_type = "sib:dependency"
+        serializer_fields = ["@id", "package", "parameters"]
+        superuser_perms = ["inherit"]
+        verbose_name = _("application package")
+        verbose_name_plural = _("application packages")
+
+    def save(self, *args, **kwargs):
+        if (
+            not self.pk
+            and ApplicationPackage.objects.filter(
+                package=self.package, application=self.application
+            ).exists()
+        ):
+            return
+
+        super(ApplicationPackage, self).save(*args, **kwargs)
+
+
+class ApplicationPackageParameter(Model):
+    package = models.ForeignKey(
+        ApplicationPackage,
+        on_delete=models.CASCADE,
+        related_name="parameters",
+        null=True,
+        blank=True,
+    )
+    key = models.CharField(max_length=255, blank=True, null=True)
+    value = models.CharField(max_length=255, blank=True, null=True)
+
+    def __str__(self):
+        try:
+            return "{} -> {} ({})".format(
+                self.package.application.friendly_name, self.package.package.friendly_name, self.urlid
+            )
+        except:
+            return self.urlid
+
+    class Meta(Model.Meta):
+        anonymous_perms = ["view"]
+        authenticated_perms = ["inherit"]
+        container_path = "application-package-parameters/"
+        owner_field = "package__application__creator"
+        owner_perms = ["inherit", "change", "delete"]
+        rdf_context = {
+            "package": "sib:dependency",
+            "key": "sib:key",
+            "value": "sib:value",
+        }
+        rdf_type = "sib:parameter"
+        serializer_fields = ["@id", "key", "value"]
+        superuser_perms = ["inherit"]
+        verbose_name = _("package parameter")
+        verbose_name_plural = _("package parameters")
+
+
 @receiver(pre_save, sender=Application)
 @receiver(pre_save, sender=ApplicationTemplate)
 def pre_save_slugify(sender, instance, **kwargs):
